<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:fo="http://www.w3.org/1999/XSL/Format"
  version="1.0">

  <xsl:import href="http://docbook.sourceforge.net/release/xsl/current/fo/docbook.xsl"/>

  <!-- Set paper size to A4 -->
  <xsl:param name="paper.type" select="'A4'"/>

  <!-- Set paper size to USletter -->
<!--  <xsl:param name="paper.type" select="'USletter'"/> -->

  <!-- draft mode? -->
  <xsl:param name="draft.mode" select="'yes'"/>
  <!-- fop does not handle pngs, so don't use an image -->
  <xsl:param name="draft.watermark.image" select="''"></xsl:param>

  <!--- Number sections -->
  <xsl:param name="section.autolabel" select="1"/>
  <xsl:param name="section.label.includes.component.label" select="1"/>

  <!-- add sections to toc up to depth 5 -->
  <xsl:param name="toc.section.depth">5</xsl:param>

  <!-- Use shade for verbatim environments -->
  <xsl:param name="shade.verbatim" select="1"></xsl:param>

  <!-- enable bookmarks -->
  <xsl:param name="fop1.extensions" select="1"></xsl:param>

  <!-- double sided printing -->
  <xsl:param name="double.sided" select="1"></xsl:param>

  <!-- use custom icons for admonition -->
  <!--
  <xsl:param name="admon.graphics" select="1"/>
  <xsl:param name="admon.graphics.extension" select="'.gif'"></xsl:param>
  <xsl:param name="admon.graphics.path">output/html/images/admon/</xsl:param>
  -->

  <!-- Index generation does not work - let's disable for now -->
  <xsl:param name="generate.index" select="0"></xsl:param>

  <!-- Better crossreferences -->
  <xsl:param name="local.l10n.xml" select="document('')"/>
  <l:i18n xmlns:l="http://docbook.sourceforge.net/xmlns/l10n/1.0">
    <l:l10n language="en">
      <l:context name="xref-number-and-title">
        <l:template name="appendix" text="Appendix %n on page %p"/>
        <l:template name="chapter" text="Chapter %n on page %p"/>
        <l:template name="section" text="Section %n on page %p"/>
        <l:template name="sect1" text="Section %n on page %p"/>
        <l:template name="sect2" text="Section %n on page %p"/>
        <l:template name="sect3" text="Section %n on page %p"/>
        <l:template name="sect4" text="Section %n on page %p"/>
        <l:template name="table" text="Table %n on page %p"/>
        <l:template name="figure" text="Figure %n on page %p"/>
        <l:template name="example" text="Example %n on page %p"/>
        <l:template name="itemizedlist" text="List %n on page %p"/>
        <l:template name="orderedlist" text="List %n on page %p"/>
        <l:template name="variablelist" text="List %n on page %p"/>
        <l:template name="procedure" text="Procedure %n on page %p"/>
        <l:template name="step" text="Step %n on page %p"/>
      </l:context>
    </l:l10n>
  </l:i18n>

  <!-- Beautify toc -->
  <xsl:template name="toc.line">
    <xsl:variable name="id">
      <xsl:call-template name="object.id"/>
    </xsl:variable>
    
    <xsl:variable name="label">
      <xsl:apply-templates select="." mode="label.markup"/>
    </xsl:variable>
    
    <fo:block text-align-last="justify"
      end-indent="{$toc.indent.width}pt"
      last-line-end-indent="-{$toc.indent.width}pt">
      <xsl:if test="(name()='chapter') or
                    (name()='appendix')">
        <xsl:attribute name="space-before">
          <xsl:text>7pt</xsl:text>
        </xsl:attribute>
        <xsl:attribute name="space-after">
          <xsl:text>2pt</xsl:text>
        </xsl:attribute>
        <xsl:attribute name="font-family">
          <xsl:text>sans-serif</xsl:text>
        </xsl:attribute>
      </xsl:if>

      <fo:inline keep-with-next.within-line="always">
        <xsl:choose>
          <xsl:when test="(name()='chapter') or
                          (name()='appendix')">
         <xsl:attribute name="font-weight">
              <xsl:text>bold</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="font-size">
              <xsl:value-of select="$body.font.master*1.2"/>
              <xsl:text>pt</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="font-family">
              <xsl:text>sans-serif</xsl:text>
            </xsl:attribute>
          </xsl:when>
          <xsl:otherwise/>
        </xsl:choose>
        <fo:basic-link internal-destination="{$id}">
          <xsl:if test="$label != ''">
            <xsl:copy-of select="$label"/>
            <xsl:value-of select="$autotoc.label.separator"/>
          </xsl:if>
          <xsl:apply-templates select="." mode="titleabbrev.markup"/>
        </fo:basic-link>
      </fo:inline>
      <fo:inline keep-together.within-line="always">
        <xsl:text> </xsl:text>
        <fo:leader leader-pattern="dots"
          leader-pattern-width="0.5em"
          leader-alignment="reference-area"
          keep-with-next.within-line="always"/>
        <xsl:text> </xsl:text>
        <fo:basic-link internal-destination="{$id}">
          <fo:page-number-citation ref-id="{$id}"/>
        </fo:basic-link>
      </fo:inline>
    </fo:block>
  </xsl:template>

  <!-- Make term of varlistentry in bold -->
  <xsl:template match="varlistentry/term">
    <fo:inline font-weight="bold">
      <xsl:apply-imports />
    </fo:inline>
  </xsl:template>

</xsl:stylesheet>
