<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" exclude-result-prefixes="k"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:k="http://inttools.suse.de/sxkeeper/schema/keeper">
	<xsl:output method="xml" indent="yes"/>
	<xsl:strip-space elements="*"/>
	
	<!-- global parameters -->
	<xsl:param name="partnercontext" select="1"/>
	<xsl:param name="productcontext" select="1"/>
	<xsl:param name="actor" select="1"/>
	<xsl:param name="description" select="1"/>
	<xsl:param name="references" select="1"/>
	<xsl:param name="eco" select="1"/>
	<xsl:param name="documentationimpact" select="1"/>
	<xsl:param name="documentationstatus" select="1"/>
	<xsl:param name="effort" select="1"/>
	<xsl:param name="testcase" select="1"/>
	<xsl:param name="usecase" select="1"/>
	<xsl:param name="internalwhiteboard" select="1"/>
	<xsl:param name="discussion" select="1"/>
	<xsl:param name="internal-comments" select="1"/>
	<xsl:param name="construct-xquery" select="'!!not specified!!'"/>
	<xsl:param name="construct-date" select="'!!not specified!!'"/>

  <!-- a comma seperated list of products, that should be included in the document ... 
       the default ('all') is to include all products -->
	<xsl:param name="includedproducts" select="'all'"/>
	
	<xsl:template match="/tree">
<xsl:text disable-output-escaping="yes">
<![CDATA[

<!DOCTYPE book PUBLIC "-//OASIS//DTD DocBook XML V4.2//EN"
                         "http://www.oasis-open.org/docbook/xml/4.2/docbookx.dtd"
[
<!ENTITY version "0.01">
<!ENTITY megacopy "MEGACOPYRIGHT &copy;&reg;">
]>
]]>
</xsl:text>

<book lang="en">
	<bookinfo>
        <title>SUSE Linux Enterprise ECO</title>
        <subtitle>Engineering Change Order for SUSE Linux Enterprise Products</subtitle>
        <titleabbrev>Feature Document</titleabbrev>
  
        <copyright>
            <year>2004-2013</year>
            <holder>SUSE Linux Products GmbH</holder>
        </copyright>

	<legalnotice>
            <para>This document is meant for SUSE internal use only.</para>
            <para>Additions done by <ulink
            url="mailto:aj@suse.com">Andreas Jaeger</ulink> and others.
            </para>
        </legalnotice>
		
        <abstract>
	  <para>
		This document was generated at <xsl:value-of select="$construct-date"/>.
	  </para>
	  <para>
		  Following query was used to generate the document:
		<emphasis><xsl:value-of select="$construct-xquery"/></emphasis>
	  </para>
          <para>
	  	This document describes features for SUSE LINUX based products
		that have been requested and approved through the ECO process,
		outside of a regular product interlock.
		It is intended to be archived for future reference.
	  </para>
	  <para>
		The features are intended to be released either as part of a
		product release, or as maintenance updates.
	  </para>
	  <para>
	    <warning><para>The products and features listed in this
                document are tentative; SUSE may or may not actually deliver
                any of these.
                </para><para>
                <emphasis>This document as a whole must not be distributed
                outside of SUSE.</emphasis>
                </para><para>
                Those parts relevant to requests by a specific partner
                may be provided to that partner upon <emphasis>explicit
                approval</emphasis> of a project manager or product manager, and must not
                include internal data like task assignments. 
                </para>
            </warning></para>

			<!-- legal stuff -->
			<para><emphasis role="bold">
			General Disclaimer
			</emphasis></para>
			<para>

			This document is not to be construed as a promise by any participating
			company to develop, deliver, or market a feature or a product.
			SUSE LINUX Products GmbH makes no representations or warranties with
			respect to the contents of this document, and specifically disclaims
			any express or implied warranties of merchantability or fitness or any
			particular purpose.  Further, SUSE LINUX Products GmbH reserves the
			right to revise this document and to make changes to its content, at
			any time, without obligation to notify any person or entity of such
			revisions or changes.

			</para>

			<xsl:choose>
			  <xsl:when test="$internal-comments = 1">
			  <!-- legal stuff -->
				<para><emphasis role="bold">
					Unpublished Work of SUSE LINUX Products GmbH.  All Rights Reserved.
				</emphasis></para>
				<para>
					This work is an unpublished work and contains confidential,
					proprietary, and trade secret information of SUSE LINUX Products GmbH.
					Access to this work is restricted to SUSE employees who have a need to
					know to perform tasks within the scope of their assignments.  No part
					of this work may be practiced, performed, copied, distributed,
					revised, modified, translated, abridged, condensed, expanded,
					collected, or adapted without the prior written consent of SUSE LINUX
					Products GmbH.  Any use or exploitation of this work without
					authorization could subject the perpetrator to criminal and civil
					liability.
				</para>
			  </xsl:when>
			  <xsl:otherwise>
				<!-- legal stuff -->
				<para><emphasis role="bold">
					Unpublished Work of SUSE LINUX Products GmbH.  All Rights Reserved.
				</emphasis></para>
				<para>
					This work is an unpublished work and contains confidential, proprietary,
					and trade secret information of SUSE LINUX Products GmbH.
					Any use or exploitation of this work without authorization could subject
					the perpetrator to criminal and civil liability.
				</para>

			  </xsl:otherwise>
			</xsl:choose>
        </abstract>
    </bookinfo>

	<xsl:apply-templates select="node">
		<xsl:with-param name="mode">chapter</xsl:with-param>
    </xsl:apply-templates>

    <xsl:if test="feature">
        <chapter>
            <title>Uncategorized</title>
            <xsl:apply-templates select="feature"/>
        </chapter>
    </xsl:if>	
</book>
		
	</xsl:template>
	
	<xsl:template match="node">		
		<xsl:param name="mode"/>
		
		<!-- root element of the node, either chapter or section-->
		<xsl:element name="{$mode}">
			<title>
				<xsl:value-of select="@category"/>
			</title>
			
			<!-- process children -->
			<xsl:apply-templates select="intro"/>
			<xsl:apply-templates select="feature"/>
			<xsl:apply-templates select="node">
				<xsl:with-param name="mode">section</xsl:with-param>
			</xsl:apply-templates>
			
		</xsl:element>		
	</xsl:template>

	<xsl:template match="intro">
		<xsl:apply-templates select="* | comment()"/>
	</xsl:template>	
			
	
	<xsl:template match="feature">
		<xsl:element name="section">
			
			<xsl:attribute name="id">
				<xsl:text>feature_</xsl:text>
				<xsl:value-of select="@k:id"/>
			</xsl:attribute>
						
			<title id="feature_{@k:id}.title">
				[<xsl:value-of select="@k:id"/>]<xsl:value-of select="title"/>
			</title>
			
			<xsl:if test="$productcontext = 1">
				<xsl:apply-templates select="productcontext"/>
			</xsl:if>
			<xsl:if test="$actor = 1">
				<xsl:call-template name="actor-table"/>
			</xsl:if>
			<xsl:if test="$partnercontext = 1">
				<xsl:apply-templates select="partnercontext"/>
			</xsl:if>
			<xsl:if test="$description = 1">
				<xsl:apply-templates select="description"/>
			</xsl:if>
			<xsl:if test="$references = 1">
				<xsl:apply-templates select="references"/>
			</xsl:if>
			<xsl:if test="$eco = 1">
				<xsl:apply-templates select="eco"/>
			</xsl:if>
			<xsl:if test="$documentationimpact = 1">
				<xsl:apply-templates select="documentationimpact"/>
			</xsl:if>
			<xsl:if test="$documentationstatus = 1">
				<xsl:apply-templates select="documentationstatus"/>
			</xsl:if>
			<xsl:if test="$effort = 1">
				<xsl:apply-templates select="effort"/>
			</xsl:if>
			<xsl:if test="$testcase = 1">
				<xsl:apply-templates select="testcase"/>
			</xsl:if>
			<xsl:if test="$usecase = 1">
				<xsl:apply-templates select="usecase"/>
			</xsl:if>
			<xsl:if test="$internalwhiteboard = 1">
				<xsl:apply-templates select="internalwhiteboard"/>
			</xsl:if>
			<xsl:if test="$discussion = 1">
				<xsl:apply-templates select="discussion"/>
			</xsl:if>
		</xsl:element>
	</xsl:template>
		
	<xsl:template match="partnercontext">
		<informaltable>
			<tgroup cols="2" align="left" colsep="0" rowsep="0">
				<colspec colwidth="1*" colname="title"/>
				<colspec colwidth="4*" colname="data"/>
				<thead>
					<row>
						<entry namest="title" nameend="data">
							<xsl:text>Partner: </xsl:text>
							<xsl:value-of select="organization"/>
						</entry>
					</row>
				</thead>
				<tbody>
					<row>
						<entry>External ID</entry>
						<xsl:choose>
							<xsl:when test="externalid">
								<entry><xsl:value-of select="externalid"/></entry>
							</xsl:when>
							<xsl:otherwise>
								<entry>-- none --</entry>
							</xsl:otherwise>
						</xsl:choose>
					</row>
					<row>
						<entry>Benefit</entry>
						<xsl:choose>
							<xsl:when test="partnerbenefit">
								<entry><xsl:apply-templates select="partnerbenefit/richtext"/></entry>
							</xsl:when>
							<xsl:otherwise>
								<entry>-- none --</entry>
							</xsl:otherwise>
						</xsl:choose>
					</row>
					<row>
						<entry>NDA</entry>
						<xsl:choose>
							<xsl:when test="nda">
								<entry><xsl:value-of select="nda/expires"/></entry>
							</xsl:when>
							<xsl:otherwise>
								<entry>-- none --</entry>
							</xsl:otherwise>
						</xsl:choose>
					</row>
						
				</tbody>
			</tgroup>
		</informaltable>
	</xsl:template>
	
  <xsl:template match="productcontext" mode="execute">
		<informaltable>
			<tgroup cols="2" align="left" colsep="0" rowsep="0">
				<colspec colwidth="1*" colname="title"/>
				<colspec colwidth="4*" colname="data"/>
				<thead>
					<row>
						<entry namest="title" nameend="data">
							<xsl:text>Product: </xsl:text>
							<xsl:value-of select="product/name"/>
						</entry>
					</row>
				</thead>
				<tbody>
					<row>
						<entry>Status</entry>
						<entry>
							<xsl:value-of select="name(status/*[1])"/>
							<xsl:if test="descendant::role">
								<xsl:text> (</xsl:text>
									<xsl:value-of select="descendant::role/text()"/>
								<xsl:text>)</xsl:text>
							</xsl:if>
						</entry>
					</row>
					<xsl:for-each select="priority">
						<row>
							<entry>Priority</entry>
							<entry>
								<xsl:value-of select="name(*[1])"/>
								<xsl:if test="owner">
									<xsl:text> (</xsl:text>
										<xsl:value-of select="owner/role/text()"/>
									<xsl:text>)</xsl:text>
								</xsl:if>
							</entry>
						</row>
					</xsl:for-each>
					
					<xsl:apply-templates select="actor" mode="table-row"/>
					
					<xsl:if test="architecture">
						<xsl:variable name="arch">
							<xsl:for-each select="architecture">
								<xsl:value-of select="normalize-space(.)"/>
								<xsl:text>,</xsl:text>
							</xsl:for-each>
						</xsl:variable>
						<row>
							<entry>Architecture</entry>
							<!-- cut last comma -->
							<entry><xsl:value-of select="substring($arch,0,string-length($arch))"/></entry>
						</row>
					</xsl:if>
				</tbody>
			</tgroup>
		</informaltable>
  </xsl:template>
  
  <!-- This templates evaluate whether the current productcontext should be rendered
       or not. The rendering is done in the template 'productcontext, mode execute -->
	<xsl:template match="productcontext">
    <xsl:param name="productslist"><xsl:value-of select="$includedproducts"/></xsl:param>

    <xsl:variable name="currentproduct">
      <xsl:choose>
        <!-- last token of productslist -->
        <xsl:when test="string-length(substring-before($productslist, ',')) = 0">
          <xsl:value-of select="$productslist"/>
        </xsl:when>

        <!-- next token from productslist -->
        <xsl:otherwise>
          <xsl:value-of select="substring-before($productslist, ',')"/> 
        </xsl:otherwise>

      </xsl:choose>
    </xsl:variable>

    <xsl:choose>
      <xsl:when test="$includedproducts = 'all'">
        <xsl:apply-templates select="." mode="execute"/>
      </xsl:when>

      <!-- current product is in productslist -->
      <xsl:when test="$currentproduct = string(product/name)">
        <xsl:apply-templates select="." mode="execute"/>
      </xsl:when>


      <!-- call template again with remaining tokens -->
      <xsl:when test="contains($productslist, ',')">
        <xsl:apply-templates select=".">
          <xsl:with-param name="productslist">
            <xsl:value-of  select="substring-after($productslist, ',')"/>
          </xsl:with-param>
        </xsl:apply-templates>
      </xsl:when>
    </xsl:choose>

  </xsl:template>

  <xsl:template name="actor-table">
    <xsl:if test="actor">
      <informaltable>
        <tgroup cols="2" align="left" colsep="0" rowsep="0">
          <colspec colwidth="1*"/>
					<colspec colwidth="4*"/>
					<thead><row><entry>Actors</entry></row></thead>
					<tbody>
						<xsl:apply-templates select="actor" mode="table-row"/>
					</tbody>
				</tgroup>
			</informaltable>
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="actor" mode="table-row">
		<row>
			<entry><xsl:value-of select="role/text()"/></entry>
			<entry><xsl:value-of select="person/fullname"/> (<xsl:value-of select="person/email"/>)</entry>
		</row>	
	</xsl:template>

	<xsl:template match="description">
		<bridgehead renderas="sect5">Description</bridgehead>
		<xsl:apply-templates/>
	</xsl:template>

		
	<xsl:template match="references">
		<bridgehead renderas="sect5">References</bridgehead>
		<xsl:for-each select="*">
			<xsl:choose>
				<xsl:when test="internallink | externallink">
					<para>
						<xsl:apply-templates/>
					</para>
				</xsl:when>
				<xsl:otherwise>
					<xsl:apply-templates/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>
	
	<xsl:template match="internallink">
		<ulink url="{@href}"><xsl:value-of select="."/></ulink>
	</xsl:template>
	
	<xsl:template match="externallink">
		<ulink url="{@href}"><xsl:value-of select="."/></ulink>
	</xsl:template>
	
	<xsl:template match="eco">
		<bridgehead renderas="sect5">ECO</bridgehead>
		<para>
		  <informaltable>
			<tgroup cols="2" align="left">
				<colspec colwidth="2*" colname="title"/>
				<colspec colwidth="3*" colname="data"/>
				<tbody>
					<xsl:apply-templates mode="eco"/>
				</tbody>
			</tgroup>
		  </informaltable>
		</para>

		<bridgehead renderas="sect5">ECO Approvals</bridgehead>
		<para>
		  <informaltable>
			<tgroup cols="3" align="left">
				<colspec colwidth="1*" colname="role"/>
				<colspec colwidth="2*" colname="user"/>
				<colspec colwidth="1*" colname="given"/>
				<tbody>
					<xsl:apply-templates mode="eco-approvals"/>
				</tbody>
			</tgroup>
		  </informaltable>
		</para>
	</xsl:template>
	
	<xsl:template match="documentationimpact">
		<bridgehead renderas="sect5">Documentation Impact</bridgehead>
		<xsl:apply-templates/>
	</xsl:template>
	
	<xsl:template match="documentationstatus">
		<bridgehead renderas="sect5">Documentation Status</bridgehead>
		<para><emphasis><xsl:value-of select="name(*[1])"/></emphasis></para>
	</xsl:template>
	
	<xsl:template match="effort">
		<bridgehead renderas="sect5">Needed Implementation Effort</bridgehead>
		<para><emphasis><xsl:value-of select="."/></emphasis></para>
	</xsl:template>
	
	<xsl:template match="testcase">
		<bridgehead renderas="sect5">Testcase</bridgehead>
		<xsl:apply-templates/>
	</xsl:template>
	
	<xsl:template match="usecase">
		<bridgehead renderas="sect5">Usecase</bridgehead>
		<xsl:apply-templates/>
	</xsl:template>
	
	<xsl:template match="internalwhiteboard">
		<bridgehead renderas="sect5">Internal Whiteboard</bridgehead>
		<xsl:apply-templates/>
	</xsl:template>
	
	<xsl:template match="discussion">
		<xsl:choose>
			<xsl:when test="$internal-comments = 1">
				<bridgehead renderas="sect5">Discussion</bridgehead>
				<xsl:apply-templates select="comment"/>				
			</xsl:when>
			<xsl:otherwise>
				<xsl:if test="count(comment[@private='false']) > 0">
					<bridgehead renderas="sect5">Discussion</bridgehead>				
				</xsl:if>
				<xsl:apply-templates select="comment[@private='false']"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="comment">
		<informaltable frame="none">
			<tgroup cols="1" align="left">
				<thead>
					<row><entry>
						<xsl:choose>
							<xsl:when test="author">
							    <xsl:value-of select="author/person/fullname"/> 
								(<xsl:value-of select="author/person/email"/>)
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of select="@legacyinfo"/>
							</xsl:otherwise>
						</xsl:choose>
					</entry></row>
				</thead>
				<tbody>
					<row><entry><xsl:apply-templates select="richtext"/></entry></row>
				</tbody>
			</tgroup>
		</informaltable>
	</xsl:template>
	
	<xsl:template match="text">
		<para>
			<xsl:value-of select="normalize-space(.)"/>
		</para>
	</xsl:template>
	
	<xsl:template match="richtext">
		<xsl:apply-templates/>
	</xsl:template>
	
	<xsl:template match="a" name="linkage">
		<xsl:variable name="href">
			<xsl:choose>
				<xsl:when test="@href | @href = ''">
					<xsl:value-of select="."/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="@href"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:choose>
			<xsl:when test="parent::richtext">
				<para>
					<ulink url="{$href}"><xsl:value-of select="."/></ulink>
				</para>
			</xsl:when>
			<xsl:otherwise>
				<ulink url="{$href}"><xsl:value-of select="."/></ulink>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="p">
		<xsl:choose>
			<!-- ignore first para in a partnerbenefit entry because it looks bad in table cells -->
			<xsl:when test="name(../..) = 'partnerbenefit' and count(preceding-sibling::*)=0">
				<xsl:apply-templates/>
			</xsl:when>
			<!-- FIXME: ignore nested <p> tags -->
			<xsl:when test="parent::p">
				<xsl:apply-templates/>
			</xsl:when>
			<!-- don't do double <para> in lists -->
			<xsl:when test="parent::li">
				<xsl:apply-templates/>
			</xsl:when>
			<!-- don't do <para> in <emphasis> -->
			<xsl:when test="parent::b">
				<xsl:apply-templates/>
			</xsl:when>
			<!-- no <p> in <tt> -->
            <xsl:when test="parent::tt">
                <xsl:apply-templates/>
            </xsl:when>
			<!-- don't add empty paras -->
			<xsl:when test="normalize-space(.) = ''">
			</xsl:when>
			
			<xsl:otherwise>
				<para>
					<xsl:apply-templates/>
				</para>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="pre">
		<xsl:variable name="class"><xsl:value-of select="@class"/></xsl:variable>
		<xsl:choose>
			<xsl:when test="$class = ''">
				<literallayout>
					<xsl:apply-templates/>
				</literallayout>
			</xsl:when>
			<xsl:otherwise>
				<xsl:element name="{$class}">
					<xsl:apply-templates/>
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose>		
	</xsl:template>
	
	<xsl:template match="ul">
		<itemizedlist>
			<xsl:apply-templates/>
		</itemizedlist>
	</xsl:template>
	
	<xsl:template match="ol">
		<orderedlist>
			<xsl:apply-templates/>
		</orderedlist>
	</xsl:template>
	
	<xsl:template match="li">
		<listitem>
					<para>
						<xsl:apply-templates/>
					</para>
		</listitem>
	</xsl:template>
	
	<xsl:template match="em">
		<xsl:choose>
			<xsl:when test="parent::richtext">
				<para>
					<emphasis>
						<xsl:apply-templates/>
					</emphasis>				
				</para>
			</xsl:when>
			<xsl:otherwise>
				<emphasis>
					<xsl:apply-templates/>
				</emphasis>				
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="h3">
		    <para><emphasis role="boldface"><xsl:value-of select="."/></emphasis></para>
	</xsl:template>
	
	<xsl:template match="b">
		<xsl:variable name="class"><xsl:value-of select="@class"/></xsl:variable>
		<xsl:choose>
            <xsl:when test="$class = 'formalpara-title'">
                <para>
                    <emphasis role="boldface">
                        <xsl:apply-templates/>
                    </emphasis>
                </para>
            </xsl:when>
            <xsl:otherwise>
                <xsl:choose>
                    <xsl:when test="parent::richtext">
                        <para>
                            <emphasis role="boldface">
                                <xsl:apply-templates/>
                            </emphasis>
                        </para>
                    </xsl:when>
                    <xsl:when test="parent::p">
                            <emphasis role="boldface">
                                <xsl:apply-templates/>
                            </emphasis>
                    </xsl:when>
                    <xsl:otherwise>
                        <emphasis role='boldface'>
                            <xsl:apply-templates/>
                        </emphasis>
                    </xsl:otherwise>
                </xsl:choose>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
		
    <xsl:template match="tt">
        <xsl:choose>
            <xsl:when test="parent::richtext">
                <para>
                    <literal>
                        <xsl:apply-templates/>
                    </literal>
                </para>
            </xsl:when>
            <xsl:otherwise>
                <literal>
                    <xsl:apply-templates/>
                </literal>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>


	<xsl:template match="text()">
		<xsl:choose>
			<xsl:when test="../@class = 'note'"></xsl:when>
			<!--FIXME: workaround for buggy references in svn tree -->
			<xsl:when test="parent::richtext"></xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="."/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

    <xsl:template match="ecofinalized" mode="eco">
    <row>
      <entry>Finalized</entry>
      <entry>This ECO has been finalized and is ready for implementation.</entry>
    </row>
    </xsl:template>
   
    <xsl:template match="ecocomponents" mode="eco">
      <xsl:call-template name="ecorichtext">
       <xsl:with-param name="title" select="'ECO Components'"/>
      </xsl:call-template>
    </xsl:template>

    <xsl:template match="ecohowimplemented" mode="eco">
      <xsl:call-template name="ecorichtext">
        <xsl:with-param name="title" select="'How is the change implemented'"/>
      </xsl:call-template>
    </xsl:template>

    <xsl:template match="ecohowisolated" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'How isolated is the change'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecorisks" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Risks associated with this ECO'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecodescother" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Other information regarding this ECO'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecowhyneed" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Why is this needed'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecoimpactifnot" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Impact if not implemented'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecoworkaround" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Known workarounds'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecoimpactother" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Other impact of this issue'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecoisregression" mode="eco">
    <row>
      <entry>Regression</entry>
      <entry>This ECO fixes a regression.</entry>
    </row>
    </xsl:template>
   
    <xsl:template match="ecocustomers" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Customers impacted'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecofieldcontact" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Field contact for this ECO'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecokernelapichanges" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Kernel API/ABI changes'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecouserapichanges" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'User space API/ABI changes'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="econewapiadded" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'New APIs/ABIs added'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecotestcasescreated" mode="eco">
    <row>
      <entry>Test cases</entry>
      <entry>New test cases have been created for these ABI/API changes.</entry>
    </row>
    </xsl:template>
   
    <xsl:template match="ecodocchanges" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Documentation changes'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecoreleasenotes" mode="eco">
    <row>
      <entry>Release Notes</entry>
      <entry>This ECO needs to be documented in the release notes.</entry>
    </row>
    </xsl:template>
   
    <xsl:template match="ecodoceffort" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Documentation effort'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecodoceta" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Documentation ETA'"/>
    </xsl:call-template>
    </xsl:template>

    <xsl:template match="ecoadditionaltranslation" mode="eco">
    <row>
      <entry>Localization required</entry>
      <entry>This change requires additional or new translations.</entry>
    </row>
    </xsl:template>
   
    <xsl:template match="econewprograms" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'New programs added with this ECO'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecostringchanges" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Strings/dialogs that need changing'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecodeliveryperson" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Developer implementing this ECO'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecomaintenanceperson" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Maintainer(s) for this code'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecomaintenanceduration" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Maintenance duration'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecotestingteam" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Testing team'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecocodeeta" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Code ETA'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecotestcaseeta" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Test case ETA'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecotestingeta" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Test execution ETA'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecotestingother" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Other test information'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecotestingdatabaseupdated" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Has the test database been updated'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecoassistancerequired" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Assistance required from other products'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecoassistanceeffort" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Estimated effort for assistance from other product teams'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecoresourceissues" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Resource issues'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="econewrpmadded" mode="eco">
    <row>
      <entry>New RPMs</entry>
      <entry>New RPMs were added as part of this ECO.</entry>
    </row>
    </xsl:template>
   
    <xsl:template match="ecorenamerpm" mode="eco">
    <row>
      <entry>New RPMs</entry>
      <entry>RPMs were renamed as part of this ECO.</entry>
    </row>
    </xsl:template>
   
    <xsl:template match="ecoconsumescryptoapi" mode="eco">
    <row>
      <entry>Crypto</entry>
      <entry>Code consumes Crypto API.</entry>
    </row>
    </xsl:template>
   
    <xsl:template match="ecocontainscrypto" mode="eco">
    <row>
      <entry>Crypto</entry>
      <entry>Code contains cryptographic routines.</entry>
    </row>
    </xsl:template>
   
    <xsl:template match="ecobinaryonly" mode="eco">
    <row>
      <entry>Binary Only</entry>
      <entry>Part of the code is binary only.</entry>
    </row>
    </xsl:template>
   
    <xsl:template match="econeedsrbreview" mode="eco">
    <row>
      <entry>SRB Review</entry>
      <entry>This release requires a review by the Security Review Board.</entry>
    </row>
    </xsl:template>
   
    <xsl:template match="econewpackage" mode="eco">
    <row>
      <entry>License Review</entry>
      <entry>New packages were added by the ECO, and need a license review.</entry>
    </row>
    </xsl:template>
   
    <xsl:template match="ecoaffectotherproducts" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Impact on other products'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecomarketingthemes" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Marketing themes'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="ecomarketingwhentoconsider" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'When to consider marketing messages'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="econtsdocschanged" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'Changes to NTS documentation'"/>
    </xsl:call-template>
    </xsl:template>
   
    <xsl:template match="econtstransfertraining" mode="eco">
    <xsl:call-template name="ecorichtext">
     <xsl:with-param name="title" select="'NTS Transfer training'"/>
    </xsl:call-template>
    </xsl:template>

    <xsl:template match="approval" mode="eco"/>

    <xsl:template match="approval" mode="eco-approvals">
      <row>
        <entry><xsl:value-of select="actor/role"/></entry>
        <entry><xsl:value-of select="actor/person/fullname"/></entry>
        <entry><xsl:value-of select="@given"/></entry>
      </row>
    </xsl:template>
   
    <xsl:template name="ecorichtext">
     <xsl:param name="title" select="'Unnamed section'"/>
     <row>
       <entry><xsl:value-of select="$title"/></entry>
       <entry><xsl:apply-templates select="richtext"/></entry>
     </row>
    </xsl:template>

    <xsl:template name="ecoboolean">
     <xsl:param name="name" select="'dummy'"/>
     <xsl:param name="message" select="'Missing message'"/>
     <row>
      <entry>
       <xsl:value-of select="$message"/>
      </entry><entry>
       <xsl:if test="$name">yes</xsl:if>
       <xsl:if test="not($name)">no</xsl:if>
      </entry>
     </row>
    </xsl:template>

    <xsl:template match="*" mode="eco">
      <row>
        <entry>ECO field not implemented:</entry>
        <entry><xsl:value-of select="name()"/></entry>
      </row>
    </xsl:template>

    <xsl:template match="*" mode="eco-approvals"/>

</xsl:stylesheet>
