<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" exclude-result-prefixes="k"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:k="http://inttools.suse.de/sxkeeper/schema/keeper">
	<xsl:output method="xml" indent="yes"/>
	<xsl:strip-space elements="*"/>
	
	<!-- global parameters -->
	<xsl:param name="partnercontext" select="1"/>
	<xsl:param name="productcontext" select="1"/>
	<xsl:param name="actor" select="1"/>
	<xsl:param name="description" select="1"/>
	<xsl:param name="references" select="1"/>
	<xsl:param name="eco" select="1"/>
	<xsl:param name="documentationimpact" select="1"/>
	<xsl:param name="documentationstatus" select="1"/>
	<xsl:param name="effort" select="1"/>
	<xsl:param name="testcase" select="1"/>
	<xsl:param name="usecase" select="1"/>
	<xsl:param name="internalwhiteboard" select="1"/>
	<xsl:param name="discussion" select="1"/>
	<xsl:param name="internal-comments" select="1"/>
	<xsl:param name="construct-xquery" select="'!!not specified!!'"/>
	<xsl:param name="construct-date" select="'!!not specified!!'"/>

  <!-- a comma seperated list of products, that should be included in the document ... 
       the default ('all') is to include all products -->
	<xsl:param name="includedproducts" select="'all'"/>
	
	<xsl:template match="/tree">
<xsl:text disable-output-escaping="yes">
<![CDATA[

<!DOCTYPE book PUBLIC "-//OASIS//DTD DocBook XML V4.2//EN"
                         "http://www.oasis-open.org/docbook/xml/4.2/docbookx.dtd"
[
<!ENTITY version "0.01">
<!ENTITY megacopy "MEGACOPYRIGHT &copy;&reg;">
]>
]]>
</xsl:text>

<book lang="en">
	<bookinfo>
        <title>Current and Proposed Future Features of SUSE Research &amp; Development</title>
        <subtitle>Detailed Requirements description for SUSE LINUX based products</subtitle>
        <titleabbrev>Feature Document</titleabbrev>
  
        <copyright>
            <year>2004-2013</year>
            <holder>SUSE Linux Products GmbH</holder>
        </copyright>

		<legalnotice>
            <para>This document is meant for SUSE internal use only.</para>
            <para>Additions done by <ulink
            url="mailto:aj@suse.com">Andreas Jaeger</ulink> and others.
            </para>
        </legalnotice>
		
        <abstract>
		  <para>
			This document was generated at <xsl:value-of select="$construct-date"/>.
		  </para>
		  <para>
		    Following query	was used to generate the document:
			<emphasis><xsl:value-of select="$construct-xquery"/></emphasis>
		  </para>
            <para>This document describes features for SUSE LINUX based
            products.</para>
	    <para>Currently (August 2005) the immediate products are (in order of expected release)
		<itemizedlist>
		<listitem><para>SUSE Linux Enterprise Server 9 SP2 (SLES9 SP2)</para></listitem>
		<listitem><para>Novell Linux Desktop SP2 (NLD9 SP2)</para></listitem>
		<listitem><para>SUSE Linux 10.0 (SL10.0)</para></listitem>
		<listitem><para>SUSE Linux Enterprise Server 9 SP3 (SLES9 SP3)</para></listitem>
		<listitem><para>Novell Linux Desktop SP3 (NLD9 SP3)</para></listitem>
		<listitem><para>SUSE Linux Enterprise Server 10 (SLES10)</para></listitem>
		<listitem><para>Novell Linux Desktop 10 (NLD10)</para></listitem>
		</itemizedlist>
                But this document describes also proposed features for
                SUSE Linux, Novell Linux Desktop and SUSE LINUX Enterprise Server beyond those
                listed above.
	    </para>
	    <para><warning><para>The products and features listed in this
                document are tentative; SUSE may or may not actually deliver
                any of these.
                </para><para>
                <emphasis>This document as a whole must not be distributed
                outside of SUSE.</emphasis>
                </para><para>
                Those parts relevant to requests by a specific partner
                may be provided to that partner upon <emphasis>explicit
                approval</emphasis> of a project manager or product manager, and must not
                include internal data like task assignments. 
                </para>
            </warning></para>

			<!-- legal stuff -->
			<para><emphasis role="bold">
			General Disclaimer
			</emphasis></para>
			<para>

			This document is not to be construed as a promise by any participating
			company to develop, deliver, or market a feature or a product.
			SUSE LINUX Products GmbH makes no representations or warranties with
			respect to the contents of this document, and specifically disclaims
			any express or implied warranties of merchantability or fitness or any
			particular purpose.  Further, SUSE LINUX Products GmbH reserves the
			right to revise this document and to make changes to its content, at
			any time, without obligation to notify any person or entity of such
			revisions or changes.

			</para>

			<xsl:choose>
			  <xsl:when test="$internal-comments = 1">
			  <!-- legal stuff -->
				<para><emphasis role="bold">
					Unpublished Work of SUSE LINUX Products GmbH.  All Rights Reserved.
				</emphasis></para>
				<para>
					This work is an unpublished work and contains confidential,
					proprietary, and trade secret information of SUSE LINUX Products GmbH.
					Access to this work is restricted to SUSE employees who have a need to
					know to perform tasks within the scope of their assignments.  No part
					of this work may be practiced, performed, copied, distributed,
					revised, modified, translated, abridged, condensed, expanded,
					collected, or adapted without the prior written consent of SUSE LINUX
					Products GmbH.  Any use or exploitation of this work without
					authorization could subject the perpetrator to criminal and civil
					liability.
				</para>
			  </xsl:when>
			  <xsl:otherwise>
				<!-- legal stuff -->
				<para><emphasis role="bold">
					Unpublished Work of SUSE LINUX Products GmbH.  All Rights Reserved.
				</emphasis></para>
				<para>
					This work is an unpublished work and contains confidential, proprietary,
					and trade secret information of SUSE LINUX Products GmbH.
					Any use or exploitation of this work without authorization could subject
					the perpetrator to criminal and civil liability.
				</para>

			  </xsl:otherwise>
			</xsl:choose>
        </abstract>
    </bookinfo>

	<xsl:apply-templates select="node">
		<xsl:with-param name="mode">chapter</xsl:with-param>
    </xsl:apply-templates>

    <xsl:if test="feature">
        <chapter>
            <title>Uncategorized</title>
            <xsl:apply-templates select="feature"/>
        </chapter>
    </xsl:if>	
</book>
		
	</xsl:template>
	
	<xsl:template match="node">		
		<xsl:param name="mode"/>
		
		<!-- root element of the node, either chapter or section-->
		<xsl:element name="{$mode}">
			<title>
				<xsl:value-of select="@category"/>
			</title>
			
			<!-- process children -->
			<xsl:apply-templates select="intro"/>
			<xsl:apply-templates select="feature"/>
			<xsl:apply-templates select="node">
				<xsl:with-param name="mode">section</xsl:with-param>
			</xsl:apply-templates>
			
		</xsl:element>		
	</xsl:template>

	<xsl:template match="intro">
		<xsl:apply-templates select="* | comment()"/>
	</xsl:template>	
			
	
	<xsl:template match="feature">
		<xsl:element name="section">
			
			<xsl:attribute name="id">
				<xsl:text>feature_</xsl:text>
				<xsl:value-of select="@k:id"/>
			</xsl:attribute>
						
			<title id="feature_{@k:id}.title">
				[<xsl:value-of select="@k:id"/>]<xsl:value-of select="title"/>
			</title>
			
			<xsl:if test="$productcontext = 1">
				<xsl:apply-templates select="productcontext"/>
			</xsl:if>
			<xsl:if test="$actor = 1">
				<xsl:call-template name="actor-table"/>
			</xsl:if>
			<xsl:if test="$partnercontext = 1">
				<xsl:apply-templates select="partnercontext"/>
			</xsl:if>
			<xsl:if test="$description = 1">
				<xsl:apply-templates select="description"/>
			</xsl:if>
			<xsl:if test="$references = 1">
				<xsl:apply-templates select="references"/>
			</xsl:if>
			<xsl:if test="$eco = 1">
				<xsl:apply-templates select="eco"/>
			</xsl:if>
			<xsl:if test="$documentationimpact = 1">
				<xsl:apply-templates select="documentationimpact"/>
			</xsl:if>
			<xsl:if test="$documentationstatus = 1">
				<xsl:apply-templates select="documentationstatus"/>
			</xsl:if>
			<xsl:if test="$effort = 1">
				<xsl:apply-templates select="effort"/>
			</xsl:if>
			<xsl:if test="$testcase = 1">
				<xsl:apply-templates select="testcase"/>
			</xsl:if>
			<xsl:if test="$usecase = 1">
				<xsl:apply-templates select="usecase"/>
			</xsl:if>
			<xsl:if test="$internalwhiteboard = 1">
				<xsl:apply-templates select="internalwhiteboard"/>
			</xsl:if>
			<xsl:if test="$discussion = 1">
				<xsl:apply-templates select="discussion"/>
			</xsl:if>
		</xsl:element>
	</xsl:template>
		
	<xsl:template match="partnercontext">
		<informaltable>
			<tgroup cols="2" align="left" colsep="0" rowsep="0">
				<colspec colwidth="1*" colname="title"/>
				<colspec colwidth="4*" colname="data"/>
				<thead>
					<row>
						<entry namest="title" nameend="data">
							<xsl:text>Partner: </xsl:text>
							<xsl:value-of select="organization"/>
						</entry>
					</row>
				</thead>
				<tbody>
					<row>
						<entry>External ID</entry>
						<xsl:choose>
							<xsl:when test="externalid">
								<entry><xsl:value-of select="externalid"/></entry>
							</xsl:when>
							<xsl:otherwise>
								<entry>-- none --</entry>
							</xsl:otherwise>
						</xsl:choose>
					</row>
					<row>
						<entry>Benefit</entry>
						<xsl:choose>
							<xsl:when test="partnerbenefit">
								<entry><xsl:apply-templates select="partnerbenefit/richtext"/></entry>
							</xsl:when>
							<xsl:otherwise>
								<entry>-- none --</entry>
							</xsl:otherwise>
						</xsl:choose>
					</row>
					<row>
						<entry>NDA</entry>
						<xsl:choose>
							<xsl:when test="nda">
								<entry><xsl:value-of select="nda/expires"/></entry>
							</xsl:when>
							<xsl:otherwise>
								<entry>-- none --</entry>
							</xsl:otherwise>
						</xsl:choose>
					</row>
						
				</tbody>
			</tgroup>
		</informaltable>
	</xsl:template>
	
  <xsl:template match="productcontext" mode="execute">
		<informaltable>
			<tgroup cols="2" align="left" colsep="0" rowsep="0">
				<colspec colwidth="1*" colname="title"/>
				<colspec colwidth="4*" colname="data"/>
				<thead>
					<row>
						<entry namest="title" nameend="data">
							<xsl:text>Product: </xsl:text>
							<xsl:value-of select="product/name"/>
						</entry>
					</row>
				</thead>
				<tbody>
					<row>
						<entry>Status</entry>
						<entry>
							<xsl:value-of select="name(status/*[1])"/>
							<xsl:if test="descendant::role">
								<xsl:text> (</xsl:text>
									<xsl:value-of select="descendant::role/text()"/>
								<xsl:text>)</xsl:text>
							</xsl:if>
						</entry>
					</row>
					<xsl:for-each select="priority">
						<row>
							<entry>Priority</entry>
							<entry>
								<xsl:value-of select="name(*[1])"/>
								<xsl:if test="owner">
									<xsl:text> (</xsl:text>
										<xsl:value-of select="owner/role/text()"/>
									<xsl:text>)</xsl:text>
								</xsl:if>
							</entry>
						</row>
					</xsl:for-each>
					
					<xsl:apply-templates select="actor" mode="table-row"/>
					
					<xsl:if test="architecture">
						<xsl:variable name="arch">
							<xsl:for-each select="architecture">
								<xsl:value-of select="normalize-space(.)"/>
								<xsl:text>,</xsl:text>
							</xsl:for-each>
						</xsl:variable>
						<row>
							<entry>Architecture</entry>
							<!-- cut last comma -->
							<entry><xsl:value-of select="substring($arch,0,string-length($arch))"/></entry>
						</row>
					</xsl:if>
				</tbody>
			</tgroup>
		</informaltable>
  </xsl:template>
  
  <!-- This templates evaluate whether the current productcontext should be rendered
       or not. The rendering is done in the template 'productcontext, mode execute -->
	<xsl:template match="productcontext">
    <xsl:param name="productslist"><xsl:value-of select="$includedproducts"/></xsl:param>

    <xsl:variable name="currentproduct">
      <xsl:choose>
        <!-- last token of productslist -->
        <xsl:when test="string-length(substring-before($productslist, ',')) = 0">
          <xsl:value-of select="$productslist"/>
        </xsl:when>

        <!-- next token from productslist -->
        <xsl:otherwise>
          <xsl:value-of select="substring-before($productslist, ',')"/> 
        </xsl:otherwise>

      </xsl:choose>
    </xsl:variable>

    <xsl:choose>
      <xsl:when test="$includedproducts = 'all'">
        <xsl:apply-templates select="." mode="execute"/>
      </xsl:when>

      <!-- current product is in productslist -->
      <xsl:when test="$currentproduct = string(product/name)">
        <xsl:apply-templates select="." mode="execute"/>
      </xsl:when>


      <!-- call template again with remaining tokens -->
      <xsl:when test="contains($productslist, ',')">
        <xsl:apply-templates select=".">
          <xsl:with-param name="productslist">
            <xsl:value-of  select="substring-after($productslist, ',')"/>
          </xsl:with-param>
        </xsl:apply-templates>
      </xsl:when>
    </xsl:choose>

  </xsl:template>

  <xsl:template name="actor-table">
    <xsl:if test="actor">
      <informaltable>
        <tgroup cols="2" align="left" colsep="0" rowsep="0">
          <colspec colwidth="1*"/>
					<colspec colwidth="4*"/>
					<thead><row><entry>Actors</entry></row></thead>
					<tbody>
						<xsl:apply-templates select="actor" mode="table-row"/>
					</tbody>
				</tgroup>
			</informaltable>
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="actor" mode="table-row">
		<row>
			<entry><xsl:value-of select="role/text()"/></entry>
			<entry><xsl:value-of select="person/fullname"/> (<xsl:value-of select="person/email"/>)</entry>
		</row>	
	</xsl:template>

	<xsl:template match="description">
		<bridgehead renderas="sect5">Description</bridgehead>
		<xsl:apply-templates/>
	</xsl:template>

		
	<xsl:template match="references">
		<bridgehead renderas="sect5">References</bridgehead>
		<xsl:for-each select="*">
			<xsl:choose>
				<xsl:when test="internallink | externallink">
					<para>
						<xsl:apply-templates/>
					</para>
				</xsl:when>
				<xsl:otherwise>
					<xsl:apply-templates/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>
	
	<xsl:template match="internallink">
		<ulink url="{@href}"><xsl:value-of select="."/></ulink>
	</xsl:template>
	
	<xsl:template match="externallink">
		<ulink url="{@href}"><xsl:value-of select="."/></ulink>
	</xsl:template>
	
	<xsl:template match="eco">
		<bridgehead renderas="sect5">ECO</bridgehead>
		<para>
		  This feature has been covered by an ECO. Please refer to
		  clarity for the archived ECO.
		</para>
	</xsl:template>
	
	<xsl:template match="documentationimpact">
		<bridgehead renderas="sect5">Documentation Impact</bridgehead>
		<xsl:apply-templates/>
	</xsl:template>
	
	<xsl:template match="documentationstatus">
		<bridgehead renderas="sect5">Documentation Status</bridgehead>
		<para><emphasis><xsl:value-of select="name(*[1])"/></emphasis></para>
	</xsl:template>
	
	<xsl:template match="effort">
		<bridgehead renderas="sect5">Needed Implementation Effort</bridgehead>
		<para><emphasis><xsl:value-of select="."/></emphasis></para>
	</xsl:template>
	
	<xsl:template match="testcase">
		<bridgehead renderas="sect5">Testcase</bridgehead>
		<xsl:apply-templates/>
	</xsl:template>
	
	<xsl:template match="usecase">
		<bridgehead renderas="sect5">Usecase</bridgehead>
		<xsl:apply-templates/>
	</xsl:template>
	
	<xsl:template match="internalwhiteboard">
		<bridgehead renderas="sect5">Internal Whiteboard</bridgehead>
		<xsl:apply-templates/>
	</xsl:template>
	
	<xsl:template match="discussion">
		<xsl:choose>
			<xsl:when test="$internal-comments = 1">
				<bridgehead renderas="sect5">Discussion</bridgehead>
				<xsl:apply-templates select="comment"/>				
			</xsl:when>
			<xsl:otherwise>
				<xsl:if test="count(comment[@private='false']) > 0">
					<bridgehead renderas="sect5">Discussion</bridgehead>				
				</xsl:if>
				<xsl:apply-templates select="comment[@private='false']"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="comment">
		<informaltable frame="none">
			<tgroup cols="1" align="left">
				<thead>
					<row><entry>
						<xsl:choose>
							<xsl:when test="author">
							    <xsl:value-of select="author/person/fullname"/> 
								(<xsl:value-of select="author/person/email"/>)
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of select="@legacyinfo"/>
							</xsl:otherwise>
						</xsl:choose>
					</entry></row>
				</thead>
				<tbody>
					<row><entry><xsl:apply-templates select="richtext"/></entry></row>
				</tbody>
			</tgroup>
		</informaltable>
	</xsl:template>
	
	<xsl:template match="text">
		<para>
			<xsl:value-of select="normalize-space(.)"/>
		</para>
	</xsl:template>
	
	<xsl:template match="richtext">
		<xsl:apply-templates/>
	</xsl:template>
	
	<xsl:template match="a" name="linkage">
		<xsl:variable name="href">
			<xsl:choose>
				<xsl:when test="@href | @href = ''">
					<xsl:value-of select="."/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="@href"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:choose>
			<xsl:when test="parent::richtext">
				<para>
					<ulink url="{$href}"><xsl:value-of select="."/></ulink>
				</para>
			</xsl:when>
			<xsl:otherwise>
				<ulink url="{$href}"><xsl:value-of select="."/></ulink>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="p">
		<xsl:choose>
			<!-- ignore first para in a partnerbenefit entry because it looks bad in table cells -->
			<xsl:when test="name(../..) = 'partnerbenefit' and count(preceding-sibling::*)=0">
				<xsl:apply-templates/>
			</xsl:when>
			<!-- FIXME: ignore nested <p> tags -->
			<xsl:when test="parent::p">
				<xsl:apply-templates/>
			</xsl:when>
			<!-- don't do double <para> in lists -->
			<xsl:when test="parent::li">
				<xsl:apply-templates/>
			</xsl:when>
			<!-- don't do <para> in <emphasis> -->
			<xsl:when test="parent::b">
				<xsl:apply-templates/>
			</xsl:when>
			<!-- no <p> in <tt> -->
            <xsl:when test="parent::tt">
                <xsl:apply-templates/>
            </xsl:when>
			<!-- don't add empty paras -->
			<xsl:when test="normalize-space(.) = ''">
			</xsl:when>
			
			<xsl:otherwise>
				<para>
					<xsl:apply-templates/>
				</para>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="pre">
		<xsl:variable name="class"><xsl:value-of select="@class"/></xsl:variable>
		<xsl:choose>
			<xsl:when test="$class = ''">
				<literallayout>
					<xsl:apply-templates/>
				</literallayout>
			</xsl:when>
			<xsl:otherwise>
				<xsl:element name="{$class}">
					<xsl:apply-templates/>
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose>		
	</xsl:template>
	
	<xsl:template match="ul">
		<itemizedlist>
			<xsl:apply-templates/>
		</itemizedlist>
	</xsl:template>
	
	<xsl:template match="ol">
		<orderedlist>
			<xsl:apply-templates/>
		</orderedlist>
	</xsl:template>
	
	<xsl:template match="li">
		<listitem>
					<para>
						<xsl:apply-templates/>
					</para>
		</listitem>
	</xsl:template>
	
	<xsl:template match="em">
		<xsl:choose>
			<xsl:when test="parent::richtext">
				<para>
					<emphasis>
						<xsl:apply-templates/>
					</emphasis>				
				</para>
			</xsl:when>
			<xsl:otherwise>
				<emphasis>
					<xsl:apply-templates/>
				</emphasis>				
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="h3">
		    <para><emphasis role="boldface"><xsl:value-of select="."/></emphasis></para>
	</xsl:template>
	
	<xsl:template match="b">
		<xsl:variable name="class"><xsl:value-of select="@class"/></xsl:variable>
		<xsl:choose>
            <xsl:when test="$class = 'formalpara-title'">
                <para>
                    <emphasis role="boldface">
                        <xsl:apply-templates/>
                    </emphasis>
                </para>
            </xsl:when>
            <xsl:otherwise>
                <xsl:choose>
                    <xsl:when test="parent::richtext">
                        <para>
                            <emphasis role="boldface">
                                <xsl:apply-templates/>
                            </emphasis>
                        </para>
                    </xsl:when>
                    <xsl:when test="parent::p">
                            <emphasis role="boldface">
                                <xsl:apply-templates/>
                            </emphasis>
                    </xsl:when>
                    <xsl:otherwise>
                        <emphasis role='boldface'>
                            <xsl:apply-templates/>
                        </emphasis>
                    </xsl:otherwise>
                </xsl:choose>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
		
    <xsl:template match="tt">
        <xsl:choose>
            <xsl:when test="parent::richtext">
                <para>
                    <literal>
                        <xsl:apply-templates/>
                    </literal>
                </para>
            </xsl:when>
            <xsl:otherwise>
                <literal>
                    <xsl:apply-templates/>
                </literal>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>


	<xsl:template match="text()">
		<xsl:choose>
			<xsl:when test="../@class = 'note'"></xsl:when>
			<!--FIXME: workaround for buggy references in svn tree -->
			<xsl:when test="parent::richtext"></xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="."/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

</xsl:stylesheet>
